package javax.naming.ldap;

import java.io.StreamCorruptedException;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.util.Collections;
import java.util.Iterator;
import java.util.Enumeration;
import java.util.ArrayList;
import javax.naming.InvalidNameException;
import java.util.List;
import javax.naming.Name;

public class LdapName implements Name
{
    private transient List<Rdn> rdns;
    private transient String unparsed;
    private static final long serialVersionUID = -1595520034788997356L;
    
    public LdapName(String unparsed) throws InvalidNameException {
        this.unparsed = unparsed;
        this.parse();
    }
    
    public LdapName(List<Rdn> list) {
        this.rdns = new ArrayList(list.size());
        int i = 0;
        while (i < list.size()) {
            Object value = list.get(i);
            if (!(value instanceof Rdn)) {
                throw new IllegalArgumentException("Entry:" + value + "  not a valid type;list entries must be of type Rdn");
            }
            this.rdns.add((Rdn)value);
            ++i;
        }
    }
    
    private LdapName(String unparsed, List<Rdn> list, int n, int n2) {
        this.unparsed = unparsed;
        this.rdns = new ArrayList(list.subList(n, n2));
    }
    
    @Override
    public int size() {
        return this.rdns.size();
    }
    
    @Override
    public boolean isEmpty() {
        return this.rdns.isEmpty();
    }
    
    @Override
    public Enumeration<String> getAll() {
        return new Enumeration<String>() {
            
            @Override
            public boolean hasMoreElements() {
                return iter.hasNext();
            }
            
            @Override
            public String nextElement() {
                return ((Rdn)iter.next()).toString();
            }
        };
    }
    
    @Override
    public String get(int n) {
        return ((Rdn)this.rdns.get(n)).toString();
    }
    
    public Rdn getRdn(int n) {
        return (Rdn)this.rdns.get(n);
    }
    
    @Override
    public Name getPrefix(int i) {
        try {
            return new LdapName(null, this.rdns, 0, i);
        }
        catch (IllegalArgumentException ex) {
            throw new IndexOutOfBoundsException("Posn: " + i + ", Size: " + this.rdns.size());
        }
    }
    
    @Override
    public Name getSuffix(int i) {
        try {
            return new LdapName(null, this.rdns, i, this.rdns.size());
        }
        catch (IllegalArgumentException ex) {
            throw new IndexOutOfBoundsException("Posn: " + i + ", Size: " + this.rdns.size());
        }
    }
    
    @Override
    public boolean startsWith(Name name) {
        if (name == null) {
            return false;
        }
        int size = this.rdns.size();
        int size2 = name.size();
        return size >= size2 && this.matches(0, size2, name);
    }
    
    public boolean startsWith(List<Rdn> list) {
        if (list == null) {
            return false;
        }
        int size = this.rdns.size();
        int size2 = list.size();
        return size >= size2 && this.doesListMatch(0, size2, list);
    }
    
    @Override
    public boolean endsWith(Name name) {
        if (name == null) {
            return false;
        }
        int size = this.rdns.size();
        int size2 = name.size();
        return size >= size2 && this.matches(size - size2, size, name);
    }
    
    public boolean endsWith(List<Rdn> list) {
        if (list == null) {
            return false;
        }
        int size = this.rdns.size();
        int size2 = list.size();
        return size >= size2 && this.doesListMatch(size - size2, size, list);
    }
    
    private boolean doesListMatch(int i, int n, List<Rdn> list) {
        while (i < n) {
            if (!((Rdn)this.rdns.get(i)).equals(list.get(i - i))) {
                return false;
            }
            ++i;
        }
        return true;
    }
    
    private boolean matches(int i, int n, Name name) {
        if ((name instanceof LdapName)) {
            return this.doesListMatch(i, n, ((LdapName)name).rdns);
        }
        while (i < n) {
            String value = name.get(i - i);
            Rdn rdn;
            try {
                rdn = new Rfc2253Parser(value).parseRdn();
            }
            catch (InvalidNameException ex) {
                return false;
            }
            if (!rdn.equals(this.rdns.get(i))) {
                return false;
            }
            ++i;
        }
        return true;
    }
    
    @Override
    public Name addAll(Name name) throws InvalidNameException {
        return this.addAll(this.size(), name);
    }
    
    public Name addAll(List<Rdn> list) {
        return this.addAll(this.size(), list);
    }
    
    @Override
    public Name addAll(int n, Name name) throws InvalidNameException {
        this.unparsed = null;
        if ((name instanceof LdapName)) {
            this.rdns.addAll(n, ((LdapName)name).rdns);
            return this;
        }
        Enumeration<String> all = name.getAll();
        while (all.hasMoreElements()) {
            this.rdns.add(n++, new Rfc2253Parser((String)all.nextElement()).parseRdn());
        }
        return this;
    }
    
    public Name addAll(int n, List<Rdn> list) {
        this.unparsed = null;
        int i = 0;
        while (i < list.size()) {
            Object value = list.get(i);
            if (!(value instanceof Rdn)) {
                throw new IllegalArgumentException("Entry:" + value + "  not a valid type;suffix list entries must be of type Rdn");
            }
            this.rdns.add(i + n, (Rdn)value);
            ++i;
        }
        return this;
    }
    
    @Override
    public Name add(String s) throws InvalidNameException {
        return this.add(this.size(), s);
    }
    
    public Name add(Rdn rdn) {
        return this.add(this.size(), rdn);
    }
    
    @Override
    public Name add(int n, String s) throws InvalidNameException {
        this.rdns.add(n, new Rfc2253Parser(s).parseRdn());
        this.unparsed = null;
        return this;
    }
    
    public Name add(int n, Rdn rdn) {
        if (rdn == null) {
            throw new NullPointerException("Cannot set comp to null");
        }
        this.rdns.add(n, rdn);
        this.unparsed = null;
        return this;
    }
    
    @Override
    public Object remove(int n) throws InvalidNameException {
        this.unparsed = null;
        return ((Rdn)this.rdns.remove(n)).toString();
    }
    
    public List<Rdn> getRdns() {
        return Collections.unmodifiableList(this.rdns);
    }
    
    @Override
    public Object clone() {
        return new LdapName(this.unparsed, this.rdns, 0, this.rdns.size());
    }
    
    @Override
    public String toString() {
        if (this.unparsed != null) {
            return this.unparsed;
        }
        StringBuilder sb = new StringBuilder();
        int size = this.rdns.size();
        if (size - 1 >= 0) {
            sb.append(this.rdns.get(size - 1));
        }
        for (int i = size - 2; i >= 0; --i) {
            sb.append(',');
            sb.append(this.rdns.get(i));
        }
        return this.unparsed = sb.toString();
    }
    
    @Override
    public boolean equals(Object o) {
        if (o == this) {
            return true;
        }
        if (!(o instanceof LdapName)) {
            return false;
        }
        LdapName ldapName = (LdapName)o;
        if (this.rdns.size() != ldapName.rdns.size()) {
            return false;
        }
        if (this.unparsed != null && this.unparsed.equalsIgnoreCase(ldapName.unparsed)) {
            return true;
        }
        int i = 0;
        while (i < this.rdns.size()) {
            if (!((Rdn)this.rdns.get(i)).equals((Rdn)ldapName.rdns.get(i))) {
                return false;
            }
            ++i;
        }
        return true;
    }
    
    @Override
    public int compareTo(Object o) {
        if (!(o instanceof LdapName)) {
            throw new ClassCastException("The obj is not a LdapName");
        }
        if (o == this) {
            return 0;
        }
        LdapName ldapName = (LdapName)o;
        if (this.unparsed != null && this.unparsed.equalsIgnoreCase(ldapName.unparsed)) {
            return 0;
        }
        int min = Math.min(this.rdns.size(), ldapName.rdns.size());
        int i = 0;
        while (i < min) {
            int compareTo = ((Rdn)this.rdns.get(i)).compareTo((Rdn)ldapName.rdns.get(i));
            if (compareTo != 0) {
                return compareTo;
            }
            ++i;
        }
        return this.rdns.size() - ldapName.rdns.size();
    }
    
    @Override
    public int hashCode() {
        int n = 0;
        for (int i = 0; i < this.rdns.size(); ++i) {
            n += ((Rdn)this.rdns.get(i)).hashCode();
        }
        return n;
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
        objectOutputStream.writeObject(this.toString());
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        this.unparsed = (String)objectInputStream.readObject();
        try {
            this.parse();
        }
        catch (InvalidNameException ex) {
            throw new StreamCorruptedException("Invalid name: " + this.unparsed);
        }
    }
    
    private void parse() throws InvalidNameException {
        this.rdns = new Rfc2253Parser(this.unparsed).parseDn();
    }
}
